@@ -14,6 +14,5 @@
 # ==============================================================================
 """Activations package definition."""
 from tf_transformers.activations.gelu import gelu
-from tf_transformers.activations.swish import (hard_swish, identity,
-                                               simple_swish)
+from tf_transformers.activations.swish import hard_swish, identity, simple_swish
 from tf_transformers.activations.utils import get_activation
