@@ -1,5 +1,10 @@
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
+# mypy: ignore-errors
+
 # coding=utf-8
-# Copyright 2020 The tf_transformers Authors.
+# Copyright 2021 TF-Transformers Authors and The TensorFlow Authors.
+# All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,6 +17,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+# ==============================================================================
 
 import collections
 import json
@@ -22,7 +28,7 @@ import six
 import tensorflow as tf
 from absl import logging
 
-from tf_transformers.data import separate_x_y
+from tf_transformers.data.utils import separate_x_y
 
 logging.set_verbosity("INFO")
 
@@ -74,12 +80,12 @@ class TFWriter(object):
         schema,
         file_name="file",
         model_dir=None,
-        tag="dev",
+        tag="eval",
         n_files=10,
         shuffle=True,
         max_files_per_record=10000,
         overwrite=False,
-        verbose_counter=1000,
+        verbose_counter=None,
     ):
         """
         Args:
@@ -107,8 +113,7 @@ class TFWriter(object):
         # Schema Check
         self.is_schema_valid(schema)
         self.tag = tag
-
-        if tag not in ["train", "dev"]:
+        if tag not in ["train", "eval"]:
             logging.info("Unknown tag {} found".format(tag))
             raise Exception("Unknwon Tag")
 
@@ -123,6 +128,7 @@ class TFWriter(object):
 
         # we need this file to write the schema to the model_dir
         schema_file_name = "schema.json"
+        self.stats_file_name = "stats.json"
         if model_dir:
             if overwrite is False:
                 if os.path.exists(model_dir):
@@ -131,6 +137,7 @@ class TFWriter(object):
             os.makedirs(model_dir, exist_ok=True)
             self.file_name = file_name = os.path.join(model_dir, file_name.replace(".tfrecord", ""))
             schema_file_name = os.path.join(model_dir, schema_file_name)
+            self.stats_file_name = os.path.join(model_dir, self.stats_file_name)
 
         if tag == "train":
             if self.shuffle:
@@ -169,7 +176,7 @@ class TFWriter(object):
         """
         simple schema validation check
         """
-        for k, v in schema.items():
+        for _k, v in schema.items():
             if v[0] == "var_len":
                 assert len(v) == 2
                 assert v[1] in TF_VALUE
@@ -219,10 +226,22 @@ class TFWriter(object):
         """
         parse_fn: function which should be an iterator or generator
         """
+        import time
+
         if hasattr(parse_fn, "__iter__") and not hasattr(parse_fn, "__len__"):
+            start_time = time.time()
             for entry in parse_fn:
                 self.write_record(entry)
-            logging.info("Total individual observations/examples written is {}".format(self.global_counter))
+            end_time = time.time()
+            stats = {'time_taken': "{} seconds".format(end_time - start_time), "total_records": self.global_counter}
+            # Save stats time and record
+            with open(self.stats_file_name, "w") as f:
+                json.dump(stats, f, indent=2)
+            logging.info(
+                "Total individual observations/examples written is {} in {} seconds".format(
+                    self.global_counter, end_time - start_time
+                )
+            )
             self.close_sess()
         else:
             raise ValueError("Expected `parse_fn` to be a generator/iterator ")
@@ -272,16 +291,16 @@ class TFWriter(object):
                 the_writer.write(example_proto.SerializeToString())
                 self.examples_per_record[self.current_file_name] += 1
                 self.global_counter += 1
-
-            if self.global_counter % self.verbose_counter == 0:
-                logging.info("Wrote {} tfrecods".format(self.global_counter))
+            if self.verbose_counter:
+                if self.global_counter % self.verbose_counter == 0:
+                    logging.info("Wrote {} tfrecods".format(self.global_counter))
         else:
             the_writer = self.all_writer[0]
             the_writer.write(example_proto.SerializeToString())
             self.global_counter += 1
-
-            if self.global_counter % self.verbose_counter == 0:
-                logging.info("Wrote {} tfrecods".format(self.global_counter))
+            if self.verbose_counter:
+                if self.global_counter % self.verbose_counter == 0:
+                    logging.info("Wrote {} tfrecods".format(self.global_counter))
 
 
 class TFReader(object):
@@ -291,7 +310,7 @@ class TFReader(object):
 
     """
 
-    def __init__(self, schema, tfrecord_files, shuffle_files=False, keys=[]):
+    def __init__(self, schema, tfrecord_files, shuffle_files=False, keys=[]):  # noqa
 
         if not isinstance(tfrecord_files, (list, tuple)):
             raise Exception("input must be a list or tuple of files")
@@ -344,7 +363,7 @@ class TFReader(object):
             schema_writer_dict[key] = TF_FUNC[value[1]]  # _bytes_feature
         return schema_reader_dict, schema_writer_dict
 
-    def decode_record_var(self, record, keys=[]):
+    def decode_record_var(self, record, keys=[]):  # noqa
         """Decodes a record to a TensorFlow example."""
         feature_dict = tf.io.parse_single_example(record, self.schema_reader_fn)
 
@@ -368,35 +387,56 @@ class TFReader(object):
         y_keys=None,
         shuffle=False,
         drop_remainder=False,
-        shuffle_buffer_size=10000,
+        shuffle_buffer_size=100,
         prefetch_buffer_size=100,
     ):
         """Auto Batching
 
         Args:
-            tf_dataset : TF dataset
-            x_keys (optional): List of key names. We will filter based on this.
-            y_keys (optional): List of key names.
-            shuffle (bool, optional): [description]. Defaults to False.
-            shuffle_buffer_size (int, optional): [description]. Defaults to 10000.
+            tf_dataset (tf.data.Dataset): TF dataset
+            batch_size (int): Batch Size
+            padded_values (dict): dict of key to padded values eg: {'key': tf.constant(0)}
+            padded_shapes (dict): dict of key to padded shapes eg: 'key': (None,)}
+            x_keys (list): List of key names. We will filter based on this.
+            y_keys (list): List of key names.
+            shuffle (bool):  Defaults to False.
+            shuffle_buffer_size (int):  Defaults to 100.
+            prefetch_buffer_size (int): Defaults to 100.
 
         Returns:
-            batched tf dataset
+            tf.data.Dataset: Batched
         """
         element_spec = tf_dataset.element_spec
         _padded_values = {}
         if not padded_values:
             padded_values = {}
+        if not padded_shapes:
+            padded_shapes = {}
         # sometimes we might have to have sme custom values other than 0
         for k, v in element_spec.items():
             if k in padded_values:
                 value = padded_values[k]
                 _padded_values[k] = tf.constant(value, dtype=value.dtype)
             else:
+                if v.dtype == tf.string:
+                    _padded_values[k] = tf.constant("0", dtype=v.dtype)
+                    continue
+
                 _padded_values[k] = tf.constant(0, dtype=v.dtype)
+
+        _padded_shapes = {}
+        for k, v in element_spec.items():
+            if k in padded_shapes:
+                _padded_shapes[k] = padded_shapes[k]
+            else:
+                if len(v.shape.dims) == 0:
+                    _padded_shapes[k] = []
+                else:
+                    _padded_shapes[k] = [None] * len(v.shape.dims)
+
         dataset = tf_dataset.padded_batch(
             padding_values=_padded_values,
-            padded_shapes=padded_shapes,
+            padded_shapes=_padded_shapes,
             batch_size=batch_size,
             drop_remainder=drop_remainder,
         )
@@ -406,10 +446,10 @@ class TFReader(object):
         # fmt: on
         if shuffle:
             dataset = dataset.shuffle(shuffle_buffer_size, seed=None, reshuffle_each_iteration=True)
-        dataset = dataset.prefetch(tf.data.experimental.AUTOTUNE)
+        dataset = dataset.prefetch(tf.data.AUTOTUNE)
         return dataset
 
-    def read_record(self, keys=[], auto_batch=False, **kwargs):
+    def read_record(self, keys=[], auto_batch=False, **kwargs):  # noqa
         """Read TF records
 
         Args:
@@ -432,6 +472,10 @@ class TFReader(object):
         dataset = dataset.map(decode_fn)
         # Using `ignore_errors()` will drop the element that causes an error.
         dataset = dataset.apply(tf.data.experimental.ignore_errors())  # ==> {1., 0.5, 0.2}
+        # TODO Make it optional
+        options = tf.data.Options()
+        options.experimental_distribute.auto_shard_policy = tf.data.experimental.AutoShardPolicy.DATA
+        dataset = dataset.with_options(options)
         if auto_batch:
             dataset = self.auto_batch(dataset, **kwargs)
         return dataset
@@ -467,13 +511,12 @@ if __name__ == "__main__":
 
     tf_writer = TFWriter(tf_dummy_schema, "test.tfrecord", tag="train", overwrite=True)
 
-    for i in range(10000):
+    for _i in range(10000):
         tf_writer.write_record(data_1)
 
     import glob
 
     all_files = glob.glob("test**")
-    print("All files", all_files)
     tf_reader = TFReader(tf_dummy_schema, all_files)
     dataset = tf_reader.read_record()
 
@@ -516,11 +559,10 @@ if __name__ == "__main__":
     }
 
     tf_writer = TFWriter(tf_dummy_schema, "test2.tfrecord", tag="train", overwrite=True)
-    for i in range(1000):
+    for _i in range(1000):
         tf_writer.write_record(data_1)
         tf_writer.write_record(data_2)
 
     all_files = glob.glob("test2**")
-    print("All files", all_files)
     tf_reader = TFReader(tf_dummy_schema, all_files, keys=[])
     dataset = tf_reader.read_record()
