@@ -1,5 +1,19 @@
-# pylint: disable=g-classes-have-attributes
-# from __future__ import google_type_annotations
+# coding=utf-8
+# Copyright 2021 TF-Transformers Authors and The TensorFlow Authors.
+# All Rights Reserved.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+# ==============================================================================
 from __future__ import absolute_import, division, print_function
 
 import tensorflow as tf
@@ -39,7 +53,10 @@ class T5Attention(LegacyLayer):
         num_heads,
         head_size,
         bidirectional,
+        use_auto_regressive,
+        is_training,
         create_positonal_embedding=True,
+        use_decoder=False,
         positional_buckets=32,
         dropout_rate=0.0,
         kernel_initializer="glorot_uniform",
@@ -72,10 +89,13 @@ class T5Attention(LegacyLayer):
             bias_constraint: Constraint for dense layer kernels.
         """
         kwargs["name"] = name
-        super(T5Attention, self).__init__(**kwargs)
+        super(T5Attention, self).__init__(is_training=is_training, **kwargs)
         self._num_heads = num_heads
         self._head_size = head_size
+        self._is_training = is_training
         self._bidirectional = bidirectional
+        self._use_auto_regressive = use_auto_regressive
+        self._use_decoder = use_decoder
         self._create_positonal_embedding = create_positonal_embedding
         self._positional_buckets = positional_buckets
         self._dropout_rate = dropout_rate
@@ -456,7 +476,7 @@ class T5Attention(LegacyLayer):
             )
             position_bias += positon_mask
 
-        attention_scores += position_bias
+        attention_scores += tf.cast(position_bias, dtype=tf_utils.get_dtype())
 
         attention_probs = self._masked_softmax([attention_scores, attention_mask])
         # This is actually dropping out entire tokens to attend to, which might
@@ -483,15 +503,9 @@ class T5Attention(LegacyLayer):
         cache_value=None,
         cross_decoder_mode=False,
     ):
-        if self.is_training:
-            (
-                attention_states,
-                position_bias,
-                key_tensor,
-                value_tensor,
-            ) = self.call_training(inputs, position_bias)
-            return attention_states, position_bias, key_tensor, value_tensor
-        else:
+        # For decoder this function is used for training and inference
+        if (self._is_training is False and self._use_auto_regressive) or cache_key is not None:
+
             (
                 attention_states,
                 position_bias,
@@ -499,3 +513,11 @@ class T5Attention(LegacyLayer):
                 value_tensor,
             ) = self.call_predict(inputs, position_bias, cache_key, cache_value, cross_decoder_mode)
             return attention_states, position_bias, key_tensor, value_tensor
+        else:
+            (
+                attention_states,
+                position_bias,
+                key_tensor,
+                value_tensor,
+            ) = self.call_training(inputs, position_bias)
+            return attention_states, position_bias, key_tensor, value_tensor
